/* GRVisitor.java
   ---------
   2001 (c) Dries Van Dyck
   Combinatorial Algorithms Research Group
   Department of Applied Mathematics and Computer Science
   University of Ghent
   Krijgslaan 281--S9
   B-9000 GENT Belgium
*/

/** 
 * Abstract class following the Visitor pattern for visiting objects
 * of the GR-family.
 * @see GenRecoupCoeff 
 * @see GRCompoundFactor 
 * @see GRSummation 
 * @see GRFactor 
 * @see GRPreFactor 
 * @see GRKrDelta 
 * @see GR6jSymbol
 * @author Dries.VanDyck@rug.ac.be
 */
public abstract class GRVisitor{
    /** 
     * Specific method for visiting a {@link GenRecoupCoeff} object.
     * @param grc the {@link GenRecoupCoeff} to be visited
     * @see GenRecoupCoeff
     */
    public abstract void visitGenRecoupCoeff(GenRecoupCoeff grc);

    /** 
     * Specific method for visiting a {@link GRCompoundFactor} object.
     * @param grcf the {@link GRCompoundFactor} to be visited
     * @see GRCompoundFactor
     */
    public abstract void visitGRCompoundFactor(GRCompoundFactor grcf);

    /** 
     * Specific method for visiting a {@link GRSummation} object.
     * @param grs the {@link GRSummation} to be visited
     * @see GRSummation
     */
    public abstract void visitGRSummation(GRSummation grs);

    /** 
     * Specific method for visiting a {@link GRPreFactor} object.
     * @param grpf the {@link GRPreFactor}to be visited 
     * @see GRPreFactor
     */
    public abstract void visitGRPreFactor(GRPreFactor grpf);

    /** 
     * Specific method for visiting a {@link GRKrDelta} object.
     * @param grcd the {@link GRKrDelta} to be visited
     * @see GRKrDelta
     */
    public abstract void visitGRKrDelta(GRKrDelta grcd);

    /** Specific method for visiting a {@link GR6jSymbol} object.
     * @param grpf the {@link GR6jSymbol} to be visited
     * @see GR6jSymbol
     */
    public abstract void visitGR6jSymbol(GR6jSymbol grpf);
    
    GRVisitor(){};
}
