/* Graph.java
   ---------
   2001 (c) Dries Van Dyck
   Combinatorial Algorithms Research Group
   Department of Applied Mathematics and Computer Science
   University of Ghent
   Krijgslaan 281--S9
   B-9000 GENT Belgium   
*/

/** 
 * Interface of a general graph of which the nodes are labeled with
 * positive integers in an arbitrary way.
 * @see NodeIterator
 * @author Dries.VanDyck@rug.ac.be
 */
public interface Graph{

    /** 
     * Returns the number of nodes of the underlying graph.
     * @return the number of nodes
     */
    public int nrOfNodes();

    /** 
     * Returns the number of edges of the underlying graph.
     * @return the number of edges
     */
    public int nrOfEdges();
    
    /** 
     * Returns the original order of this graph; There will be
     * <code>getNrOfNodes()</code> nodes with labels between
     * <coede>0</code> and <code>Order()</code>, the latter not
     * included.
     * @return the original order of the graph
     */
    public int order();

    /** 
     * Returns the neighbors of the node with label <code>node</code>.
     * @param node the node of which the neighbors have to be returned
     * @return the neihbors of the specified node as an array
     */
    public int [] neighbors(int node);

    /** 
     * True if nodes <code>i</code> and <code>j</code> are connected,
     * false otherwise.
     * @param i a node of the Graph
     * @param j a node of the Graph
     * @return true if <code>i</code> and <code>j</code> are
     *         connected, false otherwise 
     */
    public boolean connected(int i, int j);

    /** 
     * Returns an iterator over the nodeset starting at the node (by
     * calling the next() method) with lowest label in some
     * ordening.
     * @return a {@link NodeIterator} pointing to the first node
     * @see NodeIterator
     */
    public NodeIterator first();

    /** 
     * Returns an iterator over the nodeset starting at the node with
     * highest label (by calling the previous() method) in some
     * ordening.
     * @return a {@link NodeIterator} pointing to the last node
     * @see NodeIterator
     */
    public NodeIterator last();

    /** 
     * Returns an iterator which will return <code>node</code> 
     * by a {@link NodeIterator#next()} call.
     * @param node the node to be returned by the first 
     *             {@link NodeIterator#next()} of the returned 
     * 	           {@link NodeIterator}
     * @return a {@link NodeIterator} returning <code>node</code> by the
     *         first {@link NodeIterator#next()} call.
     * @see NodeIterator
     */
    public NodeIterator nodeIterator(int node);
}
